.sp
.tl 'DBGetRecord(3)'\fBmhdb library API\fR'DBGetRecord(3)'
.sp 3
.tl '\fBNAME\fR'''
.in +3
DBGetRecord - get a record from the database
.in -3
.sp
.tl '\fBSYNOPSIS\fR'''
.in +3
.B
.sp
C call prototype:
.br
#include db.h
.br
.br
int DBGetRecord ( 
\fBtype, string, return_record, flags\fR
)
.in +4
int   type;
.br
char  *string;
.br
char  **return_record;
.br
int   flags;
.br
.in -4
.br
.in -3
.sp
.tl '\fBINPUTS\fR'''
.in +3
\fItype\fR        One of the following: 
.in +12
DB_GET_NEXT, DB_GET_PREVIOUS, DB_GET_ANY, DB_GET_FIRST, 
or NULL;  all values are mutually exclusive
.in -12
.br
\fIstring\fR      record in the format defined by DBIntro(5)
.br
.in +12
which contains the search criteria.
.in -12
.br
\fIflags\fR       either NULL or DB_HAS_SUBFIELDS
.in +12
index file
.in -12
.in -3
.sp
.tl '\fBOUTPUTS\fR'''
.in +3
\fIreturn_record\fR  pointer to a character string pointer.
.in +12
If a record matching the search criteria is found,
a buffer is allocated and filled with the matching record and the pointer
to the buffer is returned in this argument.
.in -12
.in -3
.sp
.tl '\fBRETURN VALUES\fR'''
.in +3
\fIDB_FAILURE\fR           errors occured during search for record
.br
\fIDB_MATCH\fR             found a record
.br
\fIDB_NOMATCH\fR           no matching record found
.br
\fIDB_CANT_PARSE\fR        can't parse the record
.br
\fIDB_TOO_MANY_FIELDS\fR   record contains too many fields
.br
\fIDB_NO_DELIMITER\fR      can't find the FIELD delimiter
.br
\fIDB_NULL_STRING\fR       string is NULL and type is NULL
.br
\fIDB_BAD_TYPE\fR          caller passed a bad type value
.in -3
.sp
.tl '\fBDESCRIPTION\fR'''
.in +3
DBGetRecord() searches the database for a record that matches the specified
search criteria.  If \fItype\fR is NULL or DB_GET_ANY, then the
\fIstring\fR
passed as the second
argument is used as a set of search criteria.  Any field which is non-empty
is compared against possible matching records using the substring
capabilities of strstr().
.sp
The caller can request that fields be broken
into any number of subfields and the subfields compared by specifying
DB_HAS_SUBFIELDS as a \fIflag\fR value.
.sp
DBGetRecord() keeps a static index which allows the caller to make a single
call to DBGetRecord() (specifying DB_GET_ANY/NULL and supplying a search
string) to initialize the index, and then using the DB_GET_NEXT and
DB_GET_PREVIOUS types to retrieve the record following or record preceeding
(respectively) the originally retrieved record.  Calls which fail to find a
record reset the index to NULL so inadvertant calls using DB_GET_NEXT or
DB_GET_PREVIOUS will not return invalid results.  The \fIstring\fR argument is
ignored when DB_GET_NEXT or DB_GET_PREVIOUS are specified in a call to
DBGetRecord().
.sp
It is valid to provide a NULL pointer for the \fIreturn_record\fR argument.
In this case DBGetRecord() can be used to check for the existance of a
given record.
.sp
If a record matching the search criteria is found, and \fIreturn_record\fR
is not NULL, then
DBGetRecord() returns a pointer to storage allocated to hold the record.
It is the callers responsibility to free this storage.
.in -3
.sp
.tl '\fBEXAMPLES\fR'''
.in +3
C call interface:
.sp
int  type=DB_GET_ANY;
.br
char *string;
.br
char *record;
.br
int  flags=DB_HAS_SUBFIELDS;
.sp
 ...
.sp
/*
.br
 * Fill in the string with a valid record
.br
 */
.br
strcpy ( string, <some record> );
.sp
if ( DBGetRecord( type, string, &record, flags ) != DB_SUCCESS )
.br
{
.sp
 ...<handle condition where a matching record is not found>...
.sp
}
.sp
 ...<handle returned record> ...
.sp
free ( record );
.sp 2
.in -3
.sp
.tl '\fBNOTES\fR'''
.in +3
The format of the \fIstring\fR argument and the use of \fBfields\fR and
\fBsubfields\fR are defined by the rules supplied in
the DBIntro(5) man page.  
.sp
DBGetRecord() is actually just a simplified front end to DBSearchDB().
There is no man page for DBSearchDB(), but the inquisitive sorts will
figure out a way to make more productive use of DBSearchDB() by looking at
its code.  Hey, its well commented.  What do you want for free?
Besides, DBSearchDB() makes use of an array structure that is generally not
accessible to application programs, not to mention its use of various index,
hash, and flat file access routines that are not part of the API.
.sp
DB_GET_ANY is not really implemented.  Its supposed to be the ability
to search the database from top to bottom.  However, there is 
something inherently evil
about doing a sequential search through the db for a record which
can match *any* field.  I'm not sure I want to deal with that 
scenario.  I wasn't that interested in database design.
.in -3
.sp
.tl '\fBBUGS\fR'''
.in +3
None known.
.in -3
.sp
.tl '\fBSEE ALSO\fR'''
.in +3
411admin(1)
.sp
Also, the database library, libmhdb.a, has a set of API man pages:
.sp
DBIntro(5), DBShutdown(3), DBParseBuf(3), DBGetRecord(3),
DBAddRecord(3), DBDeleteRecord(3)
.in -3
.sp
.tl '\fBCOPYRIGHT\fR'''
.in +3
Copyright (c) Michael J. Hammel 1995.  This program is covered by the GNU
CopyLeft, I think.  I'm not completely sure what that is.  You can
distribute copies at your leisure for non-commercial use as long as you
distribute or make readily available the source code too.
.in -3
.sp
.tl '\fBAUTHOR\fR'''
.in +3
Michael J. Hammel (mjhammel@csn.net, mjhammel@xinside.com).  I'm interested
in any comments you might have on how to improve the basic functionality of
the library.  New features would be nice but unless they are exceptionally
easy to do, I may not get around to them for awhile.  I will keep a log of
problems and make fixes to them as time permits.
.in -3
