.sp
.tl 'DBIntro(3)'\fBmhdb library API\fR'DBIntro(3)'
.sp 3
.tl '\fBNAME\fR'''
.in +3
.sp
DBIntro - introduction to the mhdb database and its API
.in -3
.sp
.tl '\fBDESCRIPTION\fR'''
.in +3
.sp
The mhdb library provides the most simplistic implementation of a indexed
sequential, hashed database file system.  It provides only 3 real
functions: add, delete, and get a record.  Updating is done through a
combination of deleting the old record and adding the new one.  This is
wasteful of disk space and in the near future another function, rebuild,
will allow a program which uses this library to recover that wasted space.
.sp
A mhdb database consists of three files:  the hash file, the index file,
and the flat file.  The flat file is composed of records of a format
described below.  Each record in the flat file has an associated record in
the index file.  Each index record is part of a two way linked list.  Index
records are also marked as active or inactive.  Active records are valid
records in the database.  Inactive records point to records in the flat
file that have been removed.  The hash file is a set of \fBvery\fR simple hash
entries which reduce lookup times on the index file.  The names of the
files have no defaults - you must specify them in the call to DBInit()
(which is the first routine you call to initialize the database - more on
this later).
.sp
.in -3
.sp
.tl '\fBFILE FORMATS\fR'''
.sp
.in +3
\fBThe Hash File\fR
.in +3
.sp
The hash file is read during database initialization and kept in memory.
In memory I refer to the hash file as a hash table.  The hash table is
modified only in memory during normal database use.
The hash table is only written to file when DBShutdown() is called, 
normally when
the program using the mhdb library is done using the database.
.sp
The hash file consists of fixed length records of 8 bytes each.  The first
4 bytes are the characters used in the hash.  The last 4 bytes are the
offset into the index file where the first record which matches this hash
is kept.  4 characters should allow a fair distribution of hashes vs db
records, but not great.  And keeping the hash in memory during execution
will also speed up searches a bit.
.sp
Originally, I didn't have this simple hash table, but lookups for even
small databases was quite slow.  This method made a marked improvement,
certainly sufficient for the limited use it was intended for (the 
411admin program).
.in -3
.sp
\fBThe Index File\fR
.sp
.in +3
The index file uses fixed length records of 20 bytes (octets) each.  The
format of an index record is as follows:
.sp
.in +3
int active     0 for active, -1 for inactive
.br
int next       offset into index file of next sequential
.in +15
record
.in -15
.br
int prev       offset into index file of previous
.in +15
sequential record
.in -15
.br
int ptr        offset into flat file of record
.in +15
associated with this
index record
.in -15
.br
int length     length in bytes of flat file record
.in -3
.sp
The \fIactive\fR field determines if the index record is currently active or if
the index is inactive (ie points to a record in the flat file that has been
deleted).  Records are not actually deleted from the mhdb database, they
are just marked as inactive.  This allows for recovery in case something
goes wrong (everyone needs a parachute when they bail out...).  In the
future, a facility to actually remove the inactive records will be
provided.  I just have to get around to writing it.
.sp
The \fInext\fR and \fIprev\fR
fields is part of the two way linked list used by the index
records.  It allows quick access to the records (if any) immediately before 
or after the current record.  
.sp
The \fIptr\fR field is the offset into the flat file for the beginning of
the actual database record associated with this index record.
.sp
The \fIlength\fR field is the number of continguous bytes (characters) 
that the database record occupies in the file.
.in -3
.sp
\fBThe Flat File\fR
.sp
.in +3
The flat file contains the actual database records - the data provided by
the application program to be filed by the mhdb library.  It has a distinct
format that, unfortunately, is not very flexible.  No tables, just one
straight flat file, with a single index field.
.sp
A flat file record consists of 11 fields.  A field is any character text
except a colon, semicolon, or newline.  The mhdb library uses colons as field
delimiters.  Although subfields can be parsed by \fIDBParseBuf()\fR for use
by the application program (this is what 411admin does with the last, first
and middle names and the phone and email fields), the mhdb itself doesn't
specifically recognize subfields.  There are some internal (non-API)
routines that understand about subfields, but I didn't make them part of
the API just to try to keep it as simple as possible.  Complain if you
want, I like it this way.  ;-)  Actually, if you want to make use of the
search routines that deal with subfields, take a look at DBSearch.c and,
more specifically, DBCompareFields().
.sp
The record is terminated with a newline so no newlines can be embedded in
the record.
.sp
The first field is normally used as the index.  I believe I wrote the
library to handle any field as the index, but I haven't tried that so make
no promises.  In fact, I don't think any of the API allows for anything but
the first field to be the index.  Anyway, the first four characters are
used to build a hash entry.  The hash entry is compared to entries in the
hash table and if one is found, it is used to retrieve the index record and
then the actual database record, if necessary.  The 411admin program uses
the last, first, and middle names to build the index field and concatenates
them together using semicolons to subdivide the field into subfields.  This
means, for very short last names, there can be semicolons in the hash.
This doesn't appear to cause problems though.  At least not that I've
found.
.in -3
.in -3
.sp
.tl '\fBBULDING A RECORD\fR'''
.in +3
.sp
There really isn't much to this.  Just make sure it contains only text
characters and that you seperate the fields with colons.  No newlines can
be embedded in there either.  The 411admin program uses the last field as a
comments field with free formatted text.  In order to prevent newlines in
the comments from being placed in the string passed to the mhdb library
routines, it converts that fields newlines to tildes.  When it retrieves a
record from the database, it converts the tildes in that field back to
newlines.
.in -3
.sp
.tl '\fBPARSING A RECORD\fR'''
.in +3
.sp
A record can be parsed into its component fields (or subfields if you
choose) using DBParseBuf().  This routine is used in many places in the
mhdb library itself and the 411admin program makes use of it to parse
records returned from the library (which are returned in one long string)
so the records can be displayed in a meaningful way.
.in -3
.sp
.tl '\fBUPDATING A RECORD\fR'''
.in +3
.sp
There is no function provided in the nhdb library specifically for
updating records.  To update a record the application must first pass the
library a copy of the record to be deleted (the fields must match exactly -
get a copy of the record using DBGetRecord()) and then provide the
"updated" record to be added.  Its a two step process and it is left to the
application program to request each step.  This might seem silly, but it
reduces the size of the library a bit and tries to keep the KISS 
paradigm - Keep It Simple, Stupid!
.in -3
.sp
.tl '\fBTYPICAL PROGRAM FLOW\fR'''
.in +3
.sp
An application would normally do the following in relation to the mhdb
database library:
.in +3
1. call DBInit() to initialize the database
.sp
1. call DBGetRecord(), DBAddRecord(), DBDelete() record,
and DBParseBuf()
during normal program operation to view, add, delete and update records.
.sp
3. call DBShutdown() to shut down the database, including flushing the hash 
table, if necessary.
.in -3
.in -3
.sp
.tl '\fBTOOLS\fR'''
.in +3
There are a couple of tools for mucking with the database without using an
application.  I created these for testing purposes, but you might find them
useful as well.  Under the tools-d directory you'll find an hred-d, rrec-d,
and an index-d directory.  The hrec-d contains programs and scripts for
viewing the hash table.  The rrec-d directory contains programs and scripts
for viewing the index table.  The index-d directory is a simple application
that allows you to take a flat file of records that are in the proper
format (as described above) and create the index and hash files associated
with it.  This would be a good base for converting other databases to the
mhdb database, although the format change utility would still have to be
written.
.in -3
.sp
.tl '\fBNOTES\fR'''
.in +3
.sp
I haven't done any testing on lookup times on the database.  I don't expect
its all that good for extremely large databases.  But for the average
name/address contact database (such as the 411admin program for which this
database was developed) or cd lists or christmas lists or whatever, its
probably sufficient.  For simple things, use simple tools.
.sp
I haven't tried indexing on anything except the first field of a record.
There is code to deal with which field is the index, but I'm not sure if it
works.  If someone tries this, let me know how it goes.  :-)
.sp
I'm sure some CS grad is going to point out in no uncertain terms that the
"hash" is not really a hash at all.  Well, la-dee-da.  You say tomato, I
say ketchup.  It works.
.in -3
.sp
.tl '\fBBUGS\fR'''
.in +3
.sp
Tons, I'm sure, but I don't know about any at this time.  Be wary of memory
leaks.  If you find any, report them.
.in -3
.sp
.tl '\fBSEE ALSO\fR'''
.in +3
.sp
411admin(1)
.sp
Also, the database library API man pages:
.sp
DBInit(5), DBShutdown(3), DBParseBuf(3), DBGetRecord(3),
DBAddRecord(3), DBDeleteRecord(3)
.in -3
.sp
.tl '\fBCOPYRIGHT\fR'''
.in +3
.sp
Copyright (c) Michael J. Hammel 1995.  This program is covered by the GNU
CopyLeft, I think.  I'm not completely sure what that is.  You can
distribute copies at your leisure for non-commercial use as long as you
distribute or make readily available the source code too.
.sp
If you're foolish enough to want to use this for a commercial application,
let me know.  I'm sure we can work something out.
.in -3
.sp
.tl '\fBAUTHOR\fR'''
.in +3
.sp
Michael J. Hammel (mjhammel@csn.net, mjhammel@xinside.com).  I'm interested
in any comments you might have on how to improve the basic functionality of
the library.  New features would be nice but unless they are exceptionally
easy to do, I may not get around to them for awhile.  I will keep a log of
problems and make fixes to them as time permits.
.in -3
