.sp
.tl 'DBParseBuf(3)'\fBmhdb library API\fR'DBParseBuf(3)'
.sp 3
.tl '\fBNAME\fR'''
.in +3
DBParseBuf - parse a field or subfield from a string
.in -3
.sp
.tl '\fBSYNOPSIS\fR'''
.in +3
.B
.sp
C call prototype:
.br
#include db.h
.br
.br
int DBParseBuf ( 
\fBoldbuf, newbuf, type, field\fR
)
.in +4
char  *oldbuf;
.br
char  **newbuf;
.br
int   type;
.br
char  *field;
.br
.in -4
.br
.in -3
.sp
.tl '\fBINPUTS\fR'''
.in +3
\fIoldbuf\fR      string to be parsed
.br
\fItype\fR        delimiter type to be used
.in +12
one of FIELD, SUBFIELD, ENDFIELD, SINGLEFIELD,
WORD, or USERFIELD
.in -12
.br
\fIfield\fR       if type is USERFIELD caller can use
.in +12
this to specify a character to use as a delimiter;
only the first character is used as a delimiter
.in -12
.in -3
.sp
.tl '\fBOUTPUTS\fR'''
.in +3
\fInewbuf\fR      where parsed string is put.  Storage 
.in +12
for string
is allocated by this routine, but the caller is
responsible for freeing it up.
.in -12
.in -3
.sp
.tl '\fBRETURN VALUES\fR'''
.in +3
\fIDB_SUCCESS\fR            string parsed successfully
.br
\fIDB_NO_DELIMITER\fR       delimiter of "type" was not
.in +22
found in oldbuf
newbuf and oldbuf are not changed
.in -22
\fIDB_UNKNOWN_DELIMITER\fR  "type" is unknown, newbuf 
.in +22
and oldbuf not changed
.in -22
\fIDB_EMPTY_FIELD\fR        the field parsed had no
.in +22
characters in it
(ie the first character in oldbuf was the
.in -22
.in -3
.sp
.tl '\fBDESCRIPTION\fR'''
.in +3
DBParseBuf() parses a \fIfield\fI from a buffer.  The \fIfield\fR 
delimiter is determined
by the \fItype\fR.  The following table shows the delimiter used based on
the type specified by the caller:
.br
         \fIType\fR             \fIDelimiter\fR
.br
         FIELD            colon (:)
.br
         SUBFIELD         semicolon (;)
.br
         ENDFIELD         newline (\n)
.br
         SINGLEFIELD      space (0x20)
.br
         WORD             space (0x20)
.br
         USERFIELD        specified by the \fIfield\fR argument
.sp
The old buffer (\fIoldbuf\fR argument) is updated in place.  In other
words, the pointer is not changed but the string the pointer points to is
changed by removing all the characters up to and including the first
delimiter.
.sp
A buffer is malloc'd for \fInewbuf\fR which is long enough to contain the
characters removed from newbuf (except the delimiter - its discarded) plus
a string termination character.  The caller is responsible for freeing up
this storage.
.in -3
.sp
.tl '\fBEXAMPLES\fR'''
.in +3
C call interface:
.sp
char oldbuf[]="This is a:test";
.br
char *newbuf;
.sp
 ...
.sp
if ( DBParseBuf( oldbuf, &newbuf, FIELD, NULL ) != DB_SUCCESS )
.br
{
.sp
 ...<handle condition where parsing failed>...
.sp
}
.sp
printf ("Field: \\"%s\\"\\n", newbuf);
.br
printf ("Old Buffer: \\"%s\\"\\n", oldbuf);
.sp
The above example would print the following:
.in +10
Field: "This is a"
.br
Old Buffer: "test"
.in -10
.sp 
.in -3
.sp
.tl '\fBNOTES\fR'''
.in +3
The oldbuf is updated with the parsed string removed.
Upon successful completion, newbuf will point to an allocated block
of storage with the parsed string.  It is the callers responsibility
to free this storage.
.sp
No, strtok() wouldn't do this.  It moved the pointer within the buffer.
Yeah, I could have figured out a way to do it with strtok(), but this was
more interesting.
.in -3
.sp
.tl '\fBBUGS\fR'''
.in +3
None known.
.in -3
.sp
.tl '\fBSEE ALSO\fR'''
.in +3
411admin(1)
.sp
Also, the database library, libmhdb.a, has a set of API man pages:
.sp
DBIntro(5), DBInit(3), DBShutdown(3), DBGetRecord(3),
DBAddRecord(3), DBDeleteRecord(3)
.in -3
.sp
.tl '\fBCOPYRIGHT\fR'''
.in +3
Copyright (c) Michael J. Hammel 1995.  This program is covered by the GNU
CopyLeft, I think.  I'm not completely sure what that is.  You can
distribute copies at your leisure for non-commercial use as long as you
distribute or make readily available the source code too.
.in -3
.sp
.tl '\fBAUTHOR\fR'''
.in +3
Michael J. Hammel (mjhammel@csn.net, mjhammel@xinside.com).  I'm interested
in any comments you might have on how to improve the basic functionality of
the library.  New features would be nice but unless they are exceptionally
easy to do, I may not get around to them for awhile.  I will keep a log of
problems and make fixes to them as time permits.
.in -3
